@@ -17,6 +17,7 @@ import ssl
 from .i18n import _
 from . import (
     error,
+    node,
     pycompat,
     util,
 )
@@ -96,13 +97,13 @@ except AttributeError:
             # in this legacy code since we don't support SNI.
 
             args = {
-                'keyfile': self._keyfile,
-                'certfile': self._certfile,
-                'server_side': server_side,
-                'cert_reqs': self.verify_mode,
-                'ssl_version': self.protocol,
-                'ca_certs': self._cacerts,
-                'ciphers': self._ciphers,
+                r'keyfile': self._keyfile,
+                r'certfile': self._certfile,
+                r'server_side': server_side,
+                r'cert_reqs': self.verify_mode,
+                r'ssl_version': self.protocol,
+                r'ca_certs': self._cacerts,
+                r'ciphers': self._ciphers,
             }
 
             return ssl.wrap_socket(socket, **args)
@@ -808,9 +809,9 @@ def validatesocket(sock):
     # If a certificate fingerprint is pinned, use it and only it to
     # validate the remote cert.
     peerfingerprints = {
-        'sha1': hashlib.sha1(peercert).hexdigest(),
-        'sha256': hashlib.sha256(peercert).hexdigest(),
-        'sha512': hashlib.sha512(peercert).hexdigest(),
+        'sha1': node.hex(hashlib.sha1(peercert).digest()),
+        'sha256': node.hex(hashlib.sha256(peercert).digest()),
+        'sha512': node.hex(hashlib.sha512(peercert).digest()),
     }
 
     def fmtfingerprint(s):
